﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
//using System.Drawing;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using Scully.Properties;
using System.Diagnostics;

namespace Scully
{
    
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public bool isInsertMode;

        public float page_width = .5f; //
        public double margin_top = 10;
        public double margin_bottom = 10;
        public double infobar_height = 34; // should get set from the fontsize * 2.5;

        public int fontSizeStep_normal = 1;
        public int fontSizeStep_big = 2;

        private bool is_loaded = false;

        private PhilStringCounter myCounter = new PhilStringCounter();
        private int charcount = 0;

        public Saving save = new Saving();
        public ScPage page;
        private TextBox textbox;
        private ScullyCaret sCaret = new ScullyCaret();
        private Grid infobar;
        private Grid backdrop;

        private FontFamily canvasFont;
        private FontFamily infobarFont;

        private string changesIndicator;

        public SolidColorBrush brushWindowBg;
        public SolidColorBrush brushCanvasFg;
        public SolidColorBrush brushCanvasBg;
        public SolidColorBrush brushCanvasHighlight;
        public SolidColorBrush brushCanvasHighlightFg;
        public SolidColorBrush brushCanvasBounds;
        public SolidColorBrush brushCaret;
        public SolidColorBrush brushInfobarFg;
        public SolidColorBrush brushInfobarBg;


        private double thicknessPageBounds;

        public MainWindow()
        {
            InitializeComponent();

            // set window title
            string beta = "";
            if (AppGlobals.abbIsBeta)
            {
                beta = " (BETA)";
            }
            this.Title = AppGlobals.appNameShort + " " + AppGlobals.appVersion + beta;

            // set title in footer
            if (AppGlobals.abbIsBeta)
            {
                beta = "b";
            }
            this.labelBarVersionInfo.Content = AppGlobals.appNameShort.ToUpper() + " " + AppGlobals.appVersionShort() + beta;

            loadPreferences();

            if (!Settings.Default.useSystemCaret)
            {  
                textbox.SelectionChanged += (sender, e) => caret_UpdatePosition();
                textbox.LostFocus += (sender, e) => sCaret.GetCaret().Visibility = Visibility.Collapsed;
                textbox.GotFocus += (sender, e) => sCaret.GetCaret().Visibility = Visibility.Visible;
            }

            textbox.TextChanged += (sender, e) => TextChanged();

            this.Loaded += (sender, e) => onLoaded();

            this.KeyDown += new KeyEventHandler(MainWindow_KeyDown);
            this.textbox.SelectionChanged += (sender, e) => bar_updateCaretPosition();

            this.SizeChanged += (sender, e) => resizeDimensions();

            // todo: setDimensions on Maximize; 
            this.SizeChanged += new SizeChangedEventHandler(MainWindow_Maximize);

            // insert key stuff
            this.PreviewKeyDown += (sender, e) => KeyboardInsertKeyHandling();
            this.GotKeyboardFocus += (sender, e) => KeyboardInsertKeyHandling();

            this.DataContext = this; // no idea what that does

            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Save, (sender, e) => performSave()));
            this.InputBindings.Add(new KeyBinding(ApplicationCommands.Save, new KeyGesture(Key.S, ModifierKeys.Control)));

            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.SaveAs, (sender, e) => performSaveAs()));
            this.InputBindings.Add(new KeyBinding(ApplicationCommands.SaveAs, new KeyGesture(Key.S, ModifierKeys.Control | ModifierKeys.Shift)));

            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Open, (sender, e) => performOpen()));
            this.InputBindings.Add(new KeyBinding(ApplicationCommands.Open, new KeyGesture(Key.O, ModifierKeys.Control)));

            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Help, (sender, e) => performHelp()));
            this.InputBindings.Add(new KeyBinding(ApplicationCommands.Help, new KeyGesture(Key.F1)));

            this.CommandBindings.Add(new CommandBinding(EditingCommands.IncreaseFontSize, (sender, e) => textFontMainIncrease(fontSizeStep_normal)));
            this.InputBindings.Add(new KeyBinding(EditingCommands.IncreaseFontSize, new KeyGesture(Key.OemPlus, ModifierKeys.Control)));
            
            this.CommandBindings.Add(new CommandBinding(EditingCommands.DecreaseFontSize, (sender, e) => textFontMainDecrease(fontSizeStep_normal)));
            this.InputBindings.Add(new KeyBinding(EditingCommands.DecreaseFontSize, new KeyGesture(Key.OemMinus, ModifierKeys.Control)));

            this.CommandBindings.Add(new CommandBinding(EditingCommands.ToggleSubscript, (sender, e) => InsertLineSingle(textbox)));
            this.InputBindings.Add(new KeyBinding(EditingCommands.ToggleSubscript, new KeyGesture(Key.OemMinus, ModifierKeys.Control | ModifierKeys.Shift)));
        }

        private void Zoom_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            bool handle = (Keyboard.Modifiers & ModifierKeys.Control) > 0;
            if (!handle)
                return;

            if (e.Delta < 0)
            {
                textFontMainDecrease(fontSizeStep_normal);
            }
            else 
            {
                textFontMainIncrease(fontSizeStep_normal);
            }
        }

        private void MainWindow1_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Middle)
            {
                resetEditorSize();
            }
        }

        void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.F2:
                {
                    InsertTextDate(textbox);
                } break;

                case Key.F3:
                {
                    InsertTextTime(textbox);
                } break;

                case Key.F4:
                {
                    // send message to App to reset prefs
                } break;

                case Key.F5:
                {
                    page.ChangeWidthInPx(20);
                    resizeDimensions();
                } break;

                case Key.F6:
                {
                    page.ChangeWidthInPx(-20);
                    resizeDimensions();
                } break;

                case Key.F8:
                {
                    Settings.Default.displayPageBounds = !Settings.Default.displayPageBounds;
                    resizeDimensions();
                } break;

                case Key.F9:
                {
                    Settings.Default.displayInfoBar = !Settings.Default.displayInfoBar;
                    resizeDimensions();
                } break;

                case Key.F11:
                {
                    // maximize window
                    if (this.WindowState != WindowState.Maximized)
                    {

                        this.WindowStyle = WindowStyle.None;
                        this.WindowState = WindowState.Maximized;

                        resizeDimensions();
                        blankWindowsCreate();
                    }
                    else
                    {
                        blankWindowsClose();

                        this.WindowState = WindowState.Normal;
                        this.WindowStyle = WindowStyle.SingleBorderWindow;

                        resizeDimensions();
                    }
                } break;
            }
        }

        void resetEditorSize()
        {
            Settings.Default.fontsizeCanvas = Settings.Default.fontsizeCanvasDefault;
            Settings.Default.fontsizeInfobar = Settings.Default.fontsizeInfobarDefault;

            resizeDimensions();
        }

        void loadPreferences()
        {
            textbox = textCanvas;
            infobar = infoBar;
            backdrop = windowPageBackground;
            page = new ScPage(textbox, this);


            // todo: move to prefs
            canvasFont = (FontFamily)FindResource("FontText");
            infobarFont = (FontFamily)FindResource("FontBar");

            thicknessPageBounds = (Double)TryFindResource("PageBoundsThicknessModifier");
        }

        private void performSave()
        {
            if (save.PerformSave(textbox.Text))
             //   MessageBox.Show("Save Successful");
            bar_updateChangesIndicator();
        }

        private void performSaveAs()
        {
            save.PerformSaveAs(textbox.Text);
            bar_updateChangesIndicator();
        }

        private void performOpen()
        {
            // needs the current text, in case we need to save it first 
            if (save.PerformOpen(textbox))            
            {
              //  MessageBox.Show("Opened File");
                save.pendingChanges = false;
            }
            else
            {
             //   MessageBox.Show("Cancel");
            }


            bar_updateChangesIndicator();
        }

        private void performHelp()
        {
            WindowHelp help = new WindowHelp();
            help.Owner = this;
            help.Show();
        }

        private void TextChanged()
        {
            save.pendingChanges = true;
            bar_updateCharCount();
            bar_updateWordCount();
            bar_updateChangesIndicator();
        }

        private void onLoaded()
        {
            //System.Diagnostics.Debug.WriteLine("On Loaded");

            if (!Settings.Default.useSystemCaret)
            {
              sCaret.SetCaret(this.rectangleCaret, textbox, this.canvasCaret);
              sCaret.SetBrush(brushCaret);
              sCaret.SetDimensions(textbox, canvasFont, Settings.Default.fontsizeCanvas);
              sCaret.Show();

              sCaret.HideSystemCaret(textbox);
            }
            

            bar_updateCaretPosition();
            bar_updateCharCount();
            bar_updateWordCount();
            
            textbox.Focus();

            is_loaded = true;
            resizeDimensions();
        }

        void KeyboardInsertKeyHandling()
        {
            if (!is_loaded)
                return;

            if (Keyboard.IsKeyToggled(Key.Insert))
            {
                sCaret.SetInsert(true);
                isInsertMode = true;
            }
            else
            {
                sCaret.SetInsert(false);
                isInsertMode = false;
            }
            sCaret.SetDimensions(textbox, canvasFont, Settings.Default.fontsizeCanvas);
        }

        void MainWindow_Maximize(object sender, SizeChangedEventArgs e)
        {
            resizeDimensions();
        }

        private void TextCanvas_PreviewDragEnter(object sender, DragEventArgs e)
        {
            bool isCorrect = true;

            if (e.Data.GetDataPresent(DataFormats.FileDrop, true) == true)
            {
                string[] filenames = (string[])e.Data.GetData(DataFormats.FileDrop, true);
                foreach (string filename in filenames)
                {
                    if (File.Exists(filename) == false)
                    {
                        isCorrect = false;
                        break;
                    }

                    // allow only text files?
                    FileInfo info = new FileInfo(filename);
                    if (info.Extension == ".*") // != ".txt")
                    {
                        isCorrect = false;
                        break;
                    }
                }
            }
            if (isCorrect == true)
                e.Effects = DragDropEffects.All;
            else
                e.Effects = DragDropEffects.None;
            e.Handled = true;
        }

        private void TextCanvas_PreviewDrop(object sender, DragEventArgs e)
        {
            string[] filenames = (string[])e.Data.GetData(DataFormats.FileDrop, true);
            foreach (string filename in filenames)

                // todo: only accept one file
                //textbox.Text += File.ReadAllText(filename);

                textbox.Text = File.ReadAllText(filename, Encoding.UTF8);

            e.Handled = true;

            // todo: check for changes to current do and ask to save

        }

        private void performInsert(TextBox textbox_, string insertionString, bool highlightNewText)
        {
            int car = textbox_.CaretIndex;

            // in insert mode overwrite
            if (isInsertMode && (textbox_.SelectedText.Length < 1))
            {
                textbox_.Select(car, insertionString.Length);
            }

            // overwrite selection
            if (textbox_.SelectedText.Length > 0)
            {
                textbox_.SelectedText = "";
            }

            if (highlightNewText)
            {
                int L = insertionString.Length;
                textbox_.Text = textbox_.Text.Insert(car, insertionString);
                textbox_.Select(car, L);
            }
            else
            {
                // insert & move caret
                textbox_.Text = textbox_.Text.Insert(car, insertionString);
                textbox_.CaretIndex = (car + insertionString.Length);
            }
        }

        private void InsertLineSingle(TextBox textbox_)
        {
            int L = textbox_.GetLineLength(textbox_.GetLineIndexFromCharacterIndex(textbox_.CaretIndex));

            string bar = "";
            for (var i = 0; i < L; i++)
            {
                bar += "-";
            }

            performInsert(textbox_, bar, true);
        }

        private void InsertTextDate(TextBox textbox_)
        {
            string date = DateTime.Now.ToString("yyyy-MM-dd") + " ";
            performInsert(textbox_, date, false);
        }

        private void InsertTextTime(TextBox textbox_)
        {
            string time = DateTime.Now.ToString("HH:mm") + " ";
            performInsert(textbox_, time, false);
        }

        public void textFontMainIncrease(int step_size)
        {
            Settings.Default.fontsizeCanvas += step_size;
            Settings.Default.fontsizeInfobar += step_size;

            resizeDimensions();
            caret_UpdatePosition();
            resizeCaret();
        }

        public void textFontMainDecrease(int step_size)
        {
            if (Settings.Default.fontsizeCanvas > 5)
            {
                Settings.Default.fontsizeCanvas -= step_size;
                Settings.Default.fontsizeInfobar -= step_size;

                resizeDimensions();
                caret_UpdatePosition();
                resizeCaret();
            }

        }

        private void resizeDimensions()
        {
            if (!is_loaded)
            {
                return;
            }

            // window background
            backdrop.Background = brushWindowBg;

            // info bar
            infobar.Background = brushInfobarBg;

            // infobar.Width = MainWindow1.Width;

            infobar.Height = Settings.Default.fontsizeInfobar * 1.2;
            foreach (Label l in infobar.Children)
            {
                l.FontSize = Settings.Default.fontsizeInfobar;
                l.FontFamily = infobarFont;
                l.Height = Settings.Default.fontsizeInfobar * 1;
                l.Foreground = brushInfobarFg;
                l.MinHeight = Settings.Default.fontsizeInfobar;
            }


            if (Settings.Default.displayInfoBar)
            {
                infobar.Visibility = Visibility.Visible;
            }
            else
            {
                infobar.Visibility = Visibility.Collapsed;
            }

            // canvas
            double page_margin = (1f - page.GetWidth() / 2);

            var linefactor = (canvasFont.LineSpacing * 1.8 * Settings.Default.fontsizeCanvas);

            if (Settings.Default.displayInfoBar)
            {
                // NEW
                page.SetHeight( page.GetMaxHeight() - infobar.Height * 2.5);
 
            }
            else
            {
                // new
                page.ResetHeight();
            }


            page.UpdateDimensions();


            if (Settings.Default.displayPageBounds)
            {
                // old
                textbox.BorderBrush = brushCanvasBounds;
                double thick = thicknessPageBounds * Settings.Default.fontsizeCanvas * .0625;
                textbox.BorderThickness = new Thickness(thick, 0, thick, 0);
            }
            else
            {
                textbox.BorderThickness = new Thickness(0, 0, 0, 0);
            }

            textbox.FontSize = Settings.Default.fontsizeCanvas;
            textbox.FontFamily = canvasFont;
            textbox.Foreground = brushCanvasFg;
            textbox.Background = brushCanvasBg;
            textbox.SelectionBrush = brushCanvasHighlight;


            // todo: this breaks stuff when called before OnLoad()
            if (!Settings.Default.useSystemCaret)
            {
                sCaret.SetDimensions(textbox, canvasFont, Settings.Default.fontsizeCanvas);
                sCaret.ResizeCanvas(canvasCaret, textbox);
                caret_UpdatePosition();
            }

        }

        private void resizeCaret()
        {
            if (!Settings.Default.useSystemCaret)
            {
                sCaret.SetDimensions(textbox, canvasFont, Settings.Default.fontsizeCanvas);
                sCaret.ResizeCanvas(canvasCaret, textbox);
            }
        }

        private void blankWindowsCreateNew()
        {

            /*
            // === OLD ATTEMPT #1
            foreach (System.Windows.Forms.Screen s in System.Windows.Forms.Screen.AllScreens)
            {
                if (!s.Primary)
                {

                    System.Drawing.Rectangle bounds = s.Bounds;

                    // create new window
                    var newWindow = new WindowBlank();

                    newWindow.WindowStartupLocation = WindowStartupLocation.Manual;
                    // converts pixels to visual units
                    Point topleft_px = new Point(bounds.Left, bounds.Top);
                    Point topleft_vis = PointFromScreen(topleft_px);
                    Point dimensions_px = new Point(bounds.Width, bounds.Height);
                    Point dimensions_vis = PointFromScreen(dimensions_px);

                    // note: the bouds returned from Monitor are relative to the Primary Screen
                    Point thisTopleft_px = new Point(this.Left, this.Top);
                    Point thisTopleft_vis = PointFromScreen(thisTopleft_px);

                    double left = topleft_vis.X; //(thisTopleft_vis.X)
                    double top = topleft_vis.Y;

                    newWindow.Left = Math.Floor(left);
                    newWindow.Top = Math.Floor(top);

                    newWindow.Width = Math.Abs(Math.Floor(dimensions_vis.X));
                    newWindow.Height = Math.Abs(Math.Floor(dimensions_vis.Y));

                    newWindow.AllowsTransparency = true; // to make it borderless
                                                         //newWindow.Focusable = true;
                    newWindow.Background = brushWindowBg;

                    newWindow.tag.Text = s.DeviceName; // i.ToString(); // debugging

                    //newWindow.BorderThickness = new Thickness(0);

                    //newWindow.ResizeMode = ResizeMode.NoResize;
                    newWindow.WindowStyle = WindowStyle.None;

                    //newWindow.WindowState = WindowState.Maximized;


                    newWindow.Visibility = Visibility.Visible;

                    newWindow.Owner = this;


                    MessageBox.Show("Primary:     " + primaryTopLeft_px.X.ToString() + " " + primaryTopLeft_px.Y.ToString() + " - " + primaryDimensions_px.X.ToString() + " " + primaryDimensions_px.Y.ToString()
                                  + "\nScreen " + i.ToString() + ":   "
                                  + newWindow.Left.ToString() + " " + newWindow.Top.ToString() + " - " + newWindow.Width.ToString() + " " + newWindow.Height.ToString());

                }
                */

            }


        private void blankWindowsCreate()
        {


            //// ATTEMPT 3 -----------------------------------------
            // http://www.nikola-breznjak.com/blog/quick-tips/maximizing-a-wpf-window-to-second-monitor/


            // collect screens
            System.Windows.Forms.Screen[] screens = System.Windows.Forms.Screen.AllScreens;

            // check if there are more than 1 screens
            if (screens.Length <= 1)
            {
                System.Diagnostics.Debug.WriteLine("Only 1 Monitor");
                return;
            }



            // sample current window for check on which screen the main window is
            double oldLeft = this.Left;
            double oldTop = this.Top;

            System.Drawing.Point sample = new System.Drawing.Point(Convert.ToInt32(this.Left + 50), Convert.ToInt32(this.Top + 50));


            for (int i = 0; i < screens.Length; i++)
            {
                System.Drawing.Rectangle screenBounds = screens[i].Bounds;
                bool active = screenBounds.Contains(sample);

                /*
                MessageBox.Show("Screen "+i.ToString()+" → "+active.ToString()+"\n"+
                                 sample.X.ToString() + ", " + sample.Y.ToString()+"\n"+
                                 screenBounds.X.ToString()+", "+screenBounds.Y.ToString());
                */

                if (!active)
                {
                    
                    //this.Left = screenBounds.Left;
                    //this.Top = screenBounds.Top;

                    // create new window
                    var newWindow = new WindowBlank();
                    newWindow.Left = screenBounds.Left;
                    newWindow.Top = screenBounds.Top;
                    newWindow.Owner = this;

                    newWindow.AllowsTransparency = true; // to make it borderless
                    newWindow.Background = brushWindowBg;


                    
                    newWindow.Show();

                   // MessageBox.Show("Screen " + i.ToString()+"\n"+ screenBounds.Left.ToString()+", "+screenBounds.Top.ToString());
                }
            }
            


            /* ATTEMPT 2
            //-----------------------------------


            // check if there are more than 1 screens
            int m = 0;
            foreach (Monitor s in Monitor.AllMonitors)
            {
                m++;
            }
            if (m <= 1)
            {
                System.Diagnostics.Debug.WriteLine("Only 1 Monitor");
                return;
            }

            // find primary monitor because all other dimensions relate to that
            Point primaryTopLeft_px;
            Point primaryDimensions_px;

            primaryTopLeft_px = new Point(0, 0);
            primaryDimensions_px = new Point(200, 400);

            // check whether screen contains main window
            Point sample = new Point(this.Left + 50, this.Top + 10);
            //sample = PointFromScreen(sample);


            Monitor activeMonitor = null;

            foreach (Monitor s in Monitor.AllMonitors)
            {
                Rect bounds = s.Bounds;

                //string debugString = bounds.X.ToString() + ", " + bounds.Y.ToString();
                //MessageBox.Show(debugString + " of sample\n"+sample.ToString());

                if (bounds.Contains(sample))
                {
                    activeMonitor = s;
                   // MessageBox.Show("Active monitor: "+activeMonitor.Name);
                    break;
                }
            }

            // check for Primary monitor
            foreach (Monitor s in Monitor.AllMonitors)
            {
                
                if (s.IsPrimary)
                {
                     // to prevent null exceptions
                    if (activeMonitor == null)
                    {
                        activeMonitor = s;
                    }

                    System.Windows.Rect bounds = s.Bounds;

                    primaryTopLeft_px = new Point(bounds.Left, bounds.Top);
                    primaryDimensions_px = new Point(bounds.Width, bounds.Height);
                    break;
                }
            }

            Point primaryTopLeft_vis = PointFromScreen(primaryTopLeft_px);
            Point primaryDimensions_vis = PointFromScreen(primaryDimensions_px);


            int i = 0;
            foreach (Monitor s in Monitor.AllMonitors)
            {

                i++;

                if (s != activeMonitor) //(s.IsPrimary)
                {
                    System.Windows.Rect bounds = s.Bounds;

                    // create new window
                    var newWindow = new WindowBlank();

                    newWindow.WindowStartupLocation = WindowStartupLocation.Manual;    
                    // converts pixels to visual units
                    Point topleft_px = new Point(bounds.Left, bounds.Top);
                    Point topleft_vis = PointFromScreen(topleft_px);
                    Point dimensions_px = new Point(bounds.Width, bounds.Height);
                    Point dimensions_vis = PointFromScreen(dimensions_px);

                    // note: the bouds returned from Monitor are relative to the Primary Screen
                    
                   // Point thisTopleft_px = new Point(this.Left, this.Top);
                   // Point thisTopleft_vis = PointFromScreen(thisTopleft_px);
                    

                    double left = topleft_vis.X;// + primaryDimensions_vis.X;
                    double top = topleft_vis.Y;// + primaryDimensions_vis.Y;

                    newWindow.Left = Math.Floor(left);
                    newWindow.Top = Math.Floor(top);

                    newWindow.Width = Math.Abs(Math.Floor(dimensions_vis.X));
                    newWindow.Height = Math.Abs(Math.Floor(dimensions_vis.Y));

                    newWindow.AllowsTransparency = true; // to make it borderless
                    newWindow.Focusable = false;
                    newWindow.Background = brushWindowBg;

                    newWindow.tag.Text = s.Name; // i.ToString(); // debugging
                    newWindow.BorderThickness = new Thickness(0);

                    newWindow.ResizeMode = ResizeMode.NoResize;
                    newWindow.WindowStyle = WindowStyle.None;

                    //newWindow.WindowState = WindowState.Maximized;
                    //newWindow.Show();

                    newWindow.Visibility = Visibility.Visible;

                    newWindow.Owner = this;

                    if (s.IsPrimary)
                    {
                        newWindow.Hide();
                    }

                    
                    
                    //MessageBox.Show("Primary:     "+primaryTopLeft_px.X.ToString()+" "+ primaryTopLeft_px.Y.ToString()+" - "+ primaryDimensions_px.X.ToString()+ " "+ primaryDimensions_px.Y.ToString()
                      //            +"\nScreen "+i.ToString()+":   "
                        //          +newWindow.Left.ToString() + " " + newWindow.Top.ToString() + " - " + newWindow.Width.ToString() + " " + newWindow.Height.ToString()
                          //        );
                    
                                  
                }

            }
            */
        }


        private void blankWindowsClose()
        {
          foreach (Window w in this.OwnedWindows)
            {
                w.Close();
            }
        }


        private void caret_UpdatePosition()
        {
            if (Settings.Default.useSystemCaret)
            {
                return;
            }

            sCaret.UpdatePosition(textbox);
        }



        private void bar_updateChangesIndicator()
        {
            if (save.pendingChanges)
            {
                changesIndicator = "*";
            }
            else
            {
                changesIndicator = " ";
            }

            bar_updateCaretPosition();
        }

        private void bar_updateCaretPosition()
        {

            // update caret position in infoBar
            string sep = ":";
            string of = "of"; // as in "line 12 of 14";

            
            string row = Convert.ToString(sCaret.GetRow(textbox));
            string column = Convert.ToString(sCaret.GetColumn(textbox));

            string total = textbox.LineCount.ToString();

            string labelContent = changesIndicator + column + sep + row + " "+of+" "+ total;

            labelCaretPosition.Content = labelContent;

            caret_UpdatePosition();
        }

        private void bar_updateCharCount()
        {
            if (countValue == null)
            {
                return;
            }
            charcount = myCounter.countCharsWithoutSpaces(textbox.Text);
            countValue.Content = charcount.ToString() ?? "0";
        }

        private void bar_updateWordCount()
        {
            if (countValue == null)
            {
                return;
            }
            charcount = myCounter.countWords(textbox.Text);
            countWordsValue.Content = charcount.ToString() ?? "0";
        }


    }
}
