﻿using Scully.Properties;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Scully
{
    /// <summary>
    /// Interaction logic for WindowHelp.xaml
    /// </summary>
    /// 

    public partial class WindowHelp : Window
    {


        Grid grid;
        Label title;
        List<HelpboxItem> items;

        public WindowHelp()
        {
            
            title = labelTitleBar;
            grid = HelpGrid;
            
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            PopulateShortcuts();
            SetAppearance();
            UpdateDimensions();
            
        }

        public static IEnumerable<T> FindVisualChildren<T>(DependencyObject depObj) where T : DependencyObject
        {
            if (depObj != null)
            {
                for (int i = 0; i < VisualTreeHelper.GetChildrenCount(depObj); i++)
                {
                    DependencyObject child = VisualTreeHelper.GetChild(depObj, i);
                    if (child != null && child is T)
                    {
                        yield return (T)child;
                    }

                    foreach (T childOfChild in FindVisualChildren<T>(child))
                    {
                        yield return childOfChild;
                    }
                }
            }
        }

        private void doClose()
        {
            Owner.Activate();
            this.Close();
        }

        private void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            doClose();
        }

        private void Window_Deactivated(object sender, EventArgs e)
        {
            doClose();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            doClose();
        }

        private void PopulateShortcuts()
        {
            items = new List<HelpboxItem>();

            items.Add(new HelpboxItem() { Shortcut = "F1", Description = "This Window" });
            items.Add(new HelpboxItem() { Shortcut = "F2", Description = "Insert Date" });
            items.Add(new HelpboxItem() { Shortcut = "F3", Description = "Insert Time" });
            items.Add(new HelpboxItem() { Shortcut = "F5", Description = "Increase editor page width" });
            items.Add(new HelpboxItem() { Shortcut = "F6", Description = "Decrease editor page width" });
            items.Add(new HelpboxItem() { Shortcut = "F8", Description = "Toggle page bounds display" });
            items.Add(new HelpboxItem() { Shortcut = "F9", Description = "Toggle info bar" });
            items.Add(new HelpboxItem() { Shortcut = "F11", Description = "Toggle Full Screen" });

            items.Add(new HelpboxItem() { Shortcut = "CTRL+S", Description = "Save current file" });
            items.Add(new HelpboxItem() { Shortcut = "CTRL+Shift+S", Description = "Save current file as" });
            items.Add(new HelpboxItem() { Shortcut = "CTRL+Plus", Description = "Increase editor font size" });
            items.Add(new HelpboxItem() { Shortcut = "CTRL+Minus", Description = "Decrease editor font size" });
            items.Add(new HelpboxItem() { Shortcut = "CTRL+Mouse-Wheel-Up", Description = "Increase editor font size"} );
            items.Add(new HelpboxItem() { Shortcut = "CTRL+Mouse-Wheel-Dn", Description = "Decrease editor font size" });
            items.Add(new HelpboxItem() { Shortcut = "MouseMiddleClick", Description = "Reset editor font size" });

            helpListBox.ItemsSource = items;
        }

        public void UpdateDimensions()
        {
            int linesListbox = items.Count;
            int linesOutside = 2;
            double fontsize = helpListBox.FontSize;
            double lineheight = fontsize * 1.4;

            this.Height = (linesListbox + linesOutside) * lineheight;
            helpListBox.Height = linesListbox * lineheight;
        }

        public void SetAppearance()
        {

            //this.grid.Background = new SolidColorBrush( classes.ColorConverter.ToMediaColor(Settings.Default.brushInfobarBg));

            this.Background = new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogBgDefault));
            this.HelpGrid.Background = new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogBg));

            helpListBox.Foreground = new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogFg));
            helpListBox.FontFamily = (FontFamily)FindResource("FontBar");
            helpListBox.FontSize = Settings.Default.fontsizeInfobar;


            foreach (Label l in FindVisualChildren<Label>(this))
            {
                l.Foreground = new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogEmphasis));
                l.FontFamily = (FontFamily)FindResource("FontBar");
                l.FontSize = Settings.Default.fontsizeInfobar;
            }

            /*
            foreach (TextBlock t in FindVisualChildren<TextBlock>(this))
            {
                t.Foreground = new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogFg));
                t.FontFamily = (FontFamily)FindResource("FontBar");
                t.FontSize = Settings.Default.fontsizeInfobar;
            }
            */

            helpListBox.Background = null;
            //helpListBox.BorderBrush = null;

        }
    }

    public class HelpboxItem
    {
        private string shortcut;
        public string Shortcut
        {
            get
            {
                return "<" + shortcut + "> ";
            }
            set
            {
                shortcut = value;
            }
        }

        public Brush ShortcutFg
        { 
            get
            {
                return new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogEmphasis));
            }
        }

        public Brush DescFg
        {
            get
            {
                return new SolidColorBrush(classes.ColorConverter.ToMediaColor(Settings.Default.brushDialogFg));
            }
        }

        public string Description { get; set; }
    }
}