﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Windows;
using System.Windows.Controls;

namespace Scully
{ 
    public class Saving
    {
        public bool pendingChanges = true;

        private string defaultExtension = ".txt";
        private string defaultFilter = "Text |*.txt";

        private string warningTitle = AppGlobals.appNameShort + " Notice";
        private string warningUnsavedChanges = "There are unsaved changes in the file. Save?";

        private string activeFilePath; 

        private string filename;

        public bool PerformSave(string textToSave)
        {
            if (activeFilePath == null)
            {
                // create active file to save to
                if (PerformSaveAs(textToSave))
                    return true;
            }
            else
            {
                if (saveActiveFile(textToSave))
                    return true;
            }

            return false;
        }

        public bool PerformSaveAs(string textToSave)
        {
            if (saveTextFileDialog())
            {
                // save active file
                if (saveActiveFile(textToSave))
                    return true;

            }
            else
            {
                // nothing has been saved
                //MessageBox.Show("Nothing has been saved");
            }

            return false;
        }

        public bool saveActiveFile(string textToSave)
        {
            if (activeFilePath == null)
            {
                // no save
                return false;
            }

            if (!File.Exists(activeFilePath))
            {
                // files does not exist anymore
            }

            File.WriteAllText(activeFilePath, textToSave, Encoding.UTF8);
            if (File.Exists(activeFilePath))
            {
                // save successful
                pendingChanges = false;
                return true;
            }

            return false;
        }

        public bool saveTextFileDialog()
        {
            Microsoft.Win32.SaveFileDialog saveDialog = new Microsoft.Win32.SaveFileDialog();

            if (activeFilePath == null)
            {
                saveDialog.FileName = Path.GetFileName(getDefaultFilename());
            }
            else
            {
                saveDialog.FileName = Path.GetFileName(activeFilePath);
            }

            saveDialog.DefaultExt = defaultExtension;
            saveDialog.Filter = defaultFilter;

            Nullable<bool> result = saveDialog.ShowDialog();

            if (result == true)
            {
                // Save document
                filename = saveDialog.FileName;
                if (filename == null)
                {
                    return false;
                }
                else
                {
                    setActiveFilepath(filename);
                    return true;
                }
            }
            else
            {
                return false; //filename = "";
            }
        }

        public bool PerformOpen(TextBox textbox)
        {
            string textToSave = textbox.Text;
            if (pendingChanges)
            {
                // Save Changes before Opening File?
                MessageBoxResult result = MessageBox.Show(warningUnsavedChanges, warningTitle, MessageBoxButton.YesNoCancel, MessageBoxImage.Warning);
                switch (result)
                {
                    case MessageBoxResult.Cancel:
                        {
                            return false;
                        }
                    case MessageBoxResult.Yes:
                        {
                            if (PerformSave(textToSave))
                            {
                                if (openTextFileDialog(textbox))
                                {
                                    return true;
                                }
                                else
                                {
                                    return false;
                                }
                            }   
                            else
                                return false;
                        }
                }
            }

            if (openTextFileDialog(textbox))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public bool openTextFileDialog(TextBox targetTextBox)
        {

            Microsoft.Win32.OpenFileDialog openDialog = new Microsoft.Win32.OpenFileDialog();

            openDialog.FileName = ""; // defaultFilename;
            openDialog.DefaultExt = defaultExtension;
            openDialog.Filter = defaultFilter;

            Nullable<bool> result = openDialog.ShowDialog();

            if (result == true)
            {
                // Open document
                filename = openDialog.FileName;
                if (filename == null)
                {
                    return false;
                }
                else
                {
                    // loading successfull, populate textbox:
                    targetTextBox.Text = File.ReadAllText(filename, Encoding.UTF8);

                    setActiveFilepath(filename);
                    return true;
                }
            }
            else
            {
                return false; //filename = "";
            }

        }

        private string getDefaultFilename()
        {
            string timedate = getDateTime(DateTime.Now);
            string name = AppGlobals.appNameInternal + "_file";
            return timedate + name;
        }
        private string getDateTime(DateTime datetime)
        {
            string date = datetime.ToString("yyyy-MM-dd");
            string time = datetime.ToString("HH-mm-ss");
            string separator = "_";

            return date + separator + time + separator;
        }

        private bool setActiveFilepath(string filename)
        {
            activeFilePath = filename;
            return true;
        }

        private string getActiveFilepath()
        {
            return activeFilePath;
        }

        /*
        private bool overwriteFile(string filePath)
        {
            if (File.Exists(filePath))
            {
                if (MessageBox.Show(warningFileExists, warningTitle, MessageBoxButton.YesNo, MessageBoxImage.Warning) == MessageBoxResult.No)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }

            return true;
        }
        */
    }
}
