﻿using Scully.Properties;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

namespace Scully
{
    enum PageWidthMode
    {
        Relative,
        Fixed
    }

    public class ScPage
    {
        private double pageWidth; // 0 to 1
        private double pageHeight; // in visual units!!
        private double marginTop;
        private double marginBottom;
        private double marginLeft;
        private double marginRight;
        private double pageWidthMinimum = .10; // page can not be smaller than 10% of window size
        private double pageWidthMaximum = .99; 
        private double pageHeightMinimum = 100; // in visual units!
        private double pageHeightMaximum;

        private ScullyCaret caret;
        public Text text;
        private TextBox textbox;
        private Window canvas;

        private PageWidthMode widthMode;

        public ScPage(TextBox textbox_, Window window_)
        {
            textbox = textbox_;
            canvas = window_;
            text = new Text(textbox_);
            pageHeight = textbox_.Height;

            marginTop = 10;
            marginBottom = 10;
            pageHeightMaximum = GetMaxHeight();

            GetWidthModeFromSettings();
            GetWidthFromSettings();
            ResetHeight();
            UpdateMargins();
            UpdateAppearance();
            UpdateDimensions();
        }

        private double Clamp(double valueToLegalize, double maximum, double minimum)
        {
            if (valueToLegalize < minimum)
            {
                return minimum;
            }

            if (valueToLegalize > maximum)
            {
                return maximum;
            }

            return valueToLegalize;
        }

        private double RelativeFromPixels(double valueInPixels)
        {
            double return_val = (valueInPixels / canvas.Width);

            return return_val;
        }

        private void GetWidthModeFromSettings()
        {
            string checkstring = Settings.Default.pageWidthMode;
            // clean-up of checkstring
            checkstring = checkstring.Trim();
            checkstring = checkstring.ToLower();

            switch (checkstring)
            {
                case "relative":
                    {
                        widthMode = PageWidthMode.Relative;
                    }
                    break;
                case "fixed":
                    {
                        widthMode = PageWidthMode.Fixed;
                    }
                    break;
                default:
                    {
                        widthMode = PageWidthMode.Relative;
                    }
                    break;
            }
        }

        private void SetWidthModeInSettings()
        {
            switch (widthMode)
            {
                case PageWidthMode.Relative:
                    {
                        Settings.Default.pageWidthMode = "relative";
                    }
                    break;
                case PageWidthMode.Fixed:
                    {
                        Settings.Default.pageWidthMode = "fixed";
                    }
                    break;
                default:
                    {
                        Settings.Default.pageWidthMode = "relative";
                    }
                    break;
            }
        }

        private void GetWidthFromSettings()
        {
            double value = Settings.Default.pageWidthRelative;
            double convertedValue = value * .01;
            SetWidth(convertedValue);
        }

        private void SetWidthInSettings()
        {
            Settings.Default.pageWidthRelative = pageWidth * 100;
        }

        public double GetHeight()
        {
            pageHeight = textbox.Height;
            return pageHeight;
        }

        public double GetMaxHeight()
        {
            // CHECK: whether it doubles the margin beause it's also set in UpdateDimensions()
            return canvas.Height - marginBottom - marginTop;
        }

        private void UpdateMargins()
        {
            double page_margin = (1f - pageWidth / 2);

            marginLeft = page_margin;
            marginRight = page_margin;

        }

        public void UpdateAppearance()
        {
            // gets appearance from settings
        }

        public void UpdateDimensions()
        {
            textbox.Width = canvas.Width * pageWidth;
            textbox.Height = pageHeight;
            textbox.Margin = new Thickness(marginLeft, marginTop, marginRight, marginBottom);
            GetHeight();

            SetWidthInSettings();
            SetWidthModeInSettings();
        }

        public double GetWidth()
        {
            return pageWidth;
        }

        public void ResetHeight()
        {
            SetHeight(GetMaxHeight());
        }

        public void SetHeight(double newHeight)
        {
            pageHeight = Clamp(newHeight, GetMaxHeight(), pageHeightMinimum);
        }

        public void SetWidth(double newWidth)
        {
            pageWidth = Clamp(newWidth, pageWidthMaximum, pageWidthMinimum);
            SetWidthInSettings();
        }

        public void ChangeWidthInPx(double change)
        {

            switch (widthMode)
            {
                case PageWidthMode.Relative:
                    {
                        // changes are in percent
                        SetWidth(pageWidth + RelativeFromPixels(change));
                    }
                    break;
                case PageWidthMode.Fixed:
                    {
                        // fixed width increase;
                        // canges are in columns
                    }
                    break;
            }
            //UpdateDimensions();
        }
    }


    public class Text
    {
        private TextBox textbox;

        public string content;

        public Text(TextBox textbox_)
        {
            textbox = textbox_;
            content = "";
            UpdateContent();
        }

        public Text(TextBox textbox_, string content_)
        {
            textbox = textbox_;
            content = content_;
            UpdateContent();
        }

        private void UpdateContent()
        {
            textbox.Text = content; 
        }
    }
}
