﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Scully
{
    class ScullyCaret
    {
        private Rectangle myCaret;
        private Brush caretBrush;
        private Canvas canvas;
        private bool isInsert = false;
        private double dpiX;
        private double dpiY;

        // constructor 
        public ScullyCaret()
        {
            //myCaret = new Rectangle();

            // get dpi
            var dpiXProperty = typeof(SystemParameters).GetProperty("DpiX", BindingFlags.NonPublic | BindingFlags.Static);
            var dpiYProperty = typeof(SystemParameters).GetProperty("Dpi", BindingFlags.NonPublic | BindingFlags.Static);

            dpiX = (int)dpiXProperty.GetValue(null, null);
            dpiY = (int)dpiYProperty.GetValue(null, null);
        }

        // methods
        public void SetCaret(Rectangle caret_, TextBox textbox_, Canvas canvas_)
        {
            myCaret = caret_;
            //canvas = canvas_;
            //canvas = new Canvas();
            ResizeCanvas(canvas_, textbox_);
        }
        public Rectangle GetCaret()
        {
            return myCaret;
        }

        public void SetBrush(Brush brush)
        {
            caretBrush = brush;
            myCaret.Fill = caretBrush;
        }
        public Brush GetBrush()
        {
            return caretBrush;
        }

        public void SetInsert(bool insert)
        {
            isInsert = insert;
        }
        public bool GetInsert()
        {
            return isInsert;
        }

        public Vector GetPosition()
        {
            Vector vec = new Vector(Canvas.GetLeft(myCaret), Canvas.GetTop(myCaret));
            return vec;
        }
        public void UpdatePosition(TextBox textbox)
        {
            var caretLocation = textbox.GetRectFromCharacterIndex(textbox.CaretIndex);

            if (!double.IsInfinity(caretLocation.X))
            {
                Canvas.SetLeft(myCaret, caretLocation.X);
            }

            if (!double.IsInfinity(caretLocation.Y))
            {
                if (isInsert)
                {
                    Canvas.SetTop(myCaret, caretLocation.Top);
                }
                else
                {
                    Canvas.SetTop(myCaret, caretLocation.Bottom);
                }
            }
        }

        public Vector GetDimensions()
        {
            Vector vec = new Vector(myCaret.Width, myCaret.Height);
            return vec;
        }
        
        public void SetDimensions(TextBox textbox, FontFamily myFont, double fontsize)
        {

            double factor = (96 / dpiX);
        
            var line      = myFont.LineSpacing * fontsize * factor;
            var baseline  = myFont.Baseline    * fontsize * factor;

            var line_big = (myFont.LineSpacing) * (fontsize * 1.01);
            var baseline_big  = (myFont.Baseline * 1.4) * (fontsize * 1.01);
            

            // todo: rework to fit
            double height, width, margin_top;

            // defaults
            height = line / 6;
            width = baseline * .66;
            margin_top = -height; // 0;// -baseline *1.5  ;// *.5;

            if (isInsert)
            {
                margin_top = -height * .25;
                height = line * 1.25;
            }

            myCaret.Width = width;
            myCaret.Height = height;
            myCaret.Margin = new Thickness(0, margin_top, 0, 0);

            UpdatePosition(textbox);
        }

        public void ResizeCanvas(Canvas canvas_, TextBox textbox_)
        { 
            if (canvas_ == null)
            {
                canvas_ = canvas;
            }



            double margin_bottom = GetCaret().Height*1.2;

            canvas_.Width = textbox_.Width;
            canvas_.Height = textbox_.Height + margin_bottom;
            canvas_.VerticalAlignment = textbox_.VerticalAlignment;
            canvas_.HorizontalAlignment = textbox_.HorizontalAlignment;

            canvas_.Margin = new Thickness (textbox_.Margin.Left,
                                            textbox_.Margin.Top,
                                            textbox_.Margin.Right,
                                            textbox_.Margin.Bottom);

            var tb_top = Canvas.GetTop(textbox_);
            var tb_left = Canvas.GetLeft(textbox_);

            Canvas.SetTop(canvas_, tb_top);
            Canvas.SetLeft(canvas_, tb_left);
        }

        public void Hide()
        {
            myCaret.Visibility = Visibility.Hidden;
        }
        
        public void Show()
        {
            myCaret.Visibility = Visibility.Visible;
        }

        public int GetRow(TextBox textbox)
        {
            return textbox.GetLineIndexFromCharacterIndex(textbox.CaretIndex) + 1;
        }
        
        public int GetColumn(TextBox textbox)
        {

            int line = textbox.GetLineIndexFromCharacterIndex(textbox.SelectionStart);
            int column = textbox.SelectionStart - textbox.GetCharacterIndexFromLineIndex(line);

            return column + 1;
        }

        public void HideSystemCaret(TextBox textbox)
        {
            textbox.CaretBrush = new SolidColorBrush(Color.FromArgb(0, 0, 0, 0));
        }
    }
}
